/****************************************************************************
** $Id:  cmds/cmds.h
** AutoQ3D a qt 3d model editor program
**
** Copyright (C) 2005 -Gonzalo Reynaga Garca. <gonzaloreynaga@yahoo.com>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public Licence
** as published by the Free Software Foundation; either version 2 
** of the License, or (at your option) any later option.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
** 
*****************************************************************************/

#ifndef CMDSHEADER_H
#define CMDSHEADER_H
#include <QTextStream>
#include <QTextEdit>
#include <QVBoxLayout>
#include <QString>
#include <math.h>
#include <QVector>
#include <QImage>
#include <QPalette>
#include <QScrollBar>
#include <QMultiHash>
#include <QTableWidget>
#include <QSizePolicy>


#include "../extra/mousedefines.h"
#include "../dialogs/properdiag.h"
#include "../extra/primitives.h"

#define CMDDRAWLINE 		1
#define CMDDRAWTRIANGLE		2
#define CMDCANCEL		3
#define CMDZOOM 		4
#define CMDPAN			5
#define CMDROTATEVIEW		6
#define CMDGRIPONLYMOVE		7
#define CMDGRIPONLYSCALE	8
#define CMDGRIPONLYMIRROR	9
#define CMDGRIPONLYSTRETCH	10
#define CMDGRIPONLYROTATE	11
#define CMDQUICKERASE		12
#define CMDSHOWPROPERTIES	13
#define CMDROTATE3D		14
#define CMDEXTRUDE		15
#define CMDARRAY		16
#define CMDMEANNORMALS		17
#define CMDSLICE		18
#define CMDCUBE			19
#define CMDDIST			20
#define CMDSWITCH		21
#define CMDTRINORMALS		22
#define CMDSNAP			23
#define CMDREVOLVE		24
#define CMDDRAWPOLY		25




struct dataformat{
	int format;
	float v[3];
}; 



class QMultiLineEdit;
class ProperDiag;

class cmds: public QTextEdit
{
	Q_OBJECT
public:
	cmds(QWidget* parent=0);
	~cmds();
	foundP rtlines[2];
	foundP currentpoint[1];
	foundP GriPoint[1];
	foundP SnaPoint[1];
	int SnapFlags[1];
	float *currentcolor;
	float rtcolor[3];
	float rtcolor1[3];
	int GripEditMode[1];
	int SnapMode[1];
	QVector<tselect> DataSV;
	QVector<d3line> DataSLV;
	QVector<d3triangle> DataSTV;
	float cscalarf[1];
	int cscalari[1];
	QStringList StrTex;

	


public slots:
	void SendText(QString str);
	void processcmd(QString str);
	void setCmdData(dataformat data);
	void setDataL(QVector<d3line> *Info,QVector<d3triangle> *Infotri,int *mousei);


private slots:
	int equal(d3triangle* tri1,int tp1,d3triangle* tri2,int tp2);
	int CmdSelectPopView();
	int CmdCheckSnap();
	void CmdCancel();
	void cmddrawline();
	void cmddrawtriangle();
	void cmdzoom();
	void cmdpan();
	void cmdrotateview();
	void cmdpopzoom();
	void cmdpoppan();
	void cmdpoprotateview();
	void cmdgriponlymove();
	void cmdgriponlyscale();
	void cmdgriponlymirror();
	void cmdgriponlystretch();
	void cmdgriponlyrotate();
	void cmdquickerase();
	void cmdshowproperties();
	void cmdrotate3d();
	void cmdextrude();
	void cmdarray();
	void cmdmeannormals();
	void cmdslice();
	void cmdcube();
	void cmddist();
	void cmdswitch();
	void cmdtrinormals();
	void cmdsnap();
	void cmdrevolve();
	void cmddrawpoly();


signals:
	void instru(const QString &);
	void SetMouseMode(int);
	void ZoomVar(float var);

private:
	QString instruction;
	QVector<d3line> *DataLV;
	QVector<d3triangle> *DataTV;
	QHash<QString,int> cmdlist;
	int step;
	QString currentcmd;
	QString SecCmd;
	int CommandActive;
	dataformat currentdata;
	float tempdataF[7];
	int tempdataI[3];
	d3triangle tempdataTRI[1];
	int popStats[3];
	int *MouseMode;
	int PopViewMode;
	int PopStep;
	ProperDiag* Props;
	

};


#endif
