/****************************************************************************
** $Id:  fgui/cadform.cpp
** AutoQ3D a qt 3d model editor program
**
** Copyright (C) 2005 -Gonzalo Reynaga Garca. <gonzaloreynaga@yahoo.com>
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public Licence
** as published by the Free Software Foundation; either version 2 
** of the License, or (at your option) any later option.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
** 
*****************************************************************************/


#include "cadform.h"
#include "md2/cadformmd2.h"
#include "dxf/cadformdxf.h"


CadForm::CadForm()
{
	numDoc=0;
	*MouseMode=MOUSEMODE_SELECT;
	filename="";
	currentcolor[0]=1.0;currentcolor[1]=1.0;currentcolor[2]=1.0;
	DataLV.resize(0);
	DataTV.resize(0);
	QSettings settings("AutoQ3D","AutoQ3D");
	ProgramPath = settings.value("MainWindow/ProgramPath").toString()+"/";
	

	ws = new QWorkspace;
	Label1 = new QLabel("Command:" );
	ecmd= new EditCmd;
	hcmd=new cmds();
	QWidget *WhVc=new QWidget;
	QVBoxLayout *hVc=new QVBoxLayout;
	QHBoxLayout *hHL=new QHBoxLayout;
	QSplitter *s1 = new QSplitter( Qt::Vertical);
	QVBoxLayout *vboxLayout=new QVBoxLayout;


	QAction * ViewPortNewAction;
	ViewPortNewAction= new QAction( QIcon(ProgramPath+"images/filenew.png"),"&New Viewport",this);
	ViewPortNewAction->setStatusTip("New Viewport");
	connect( ViewPortNewAction, SIGNAL( triggered() ) , this, SLOT( newViewPort() ) );

	QAction * fileOpenAction;
	fileOpenAction= new QAction(QIcon(ProgramPath+"images/fileopen.png"),"&Open...",this);
	fileOpenAction->setStatusTip("Open file");
	connect( fileOpenAction, SIGNAL( triggered() ) , this, SLOT( OpenFile() ) );

	QAction * fileSaveAction;
	fileSaveAction = new QAction(QIcon(ProgramPath+"images/filesave.png"), "&Save",this);
	fileSaveAction->setStatusTip("Save file");
	connect( fileSaveAction, SIGNAL( triggered() ) , this, SLOT( SaveFile() ) );
	
	QAction * fileCloseAction;
	fileCloseAction = new QAction( QIcon(ProgramPath+"images/doclose.png"), "&Close",this);
	fileCloseAction->setStatusTip("Close file");
	connect( fileCloseAction, SIGNAL( triggered() ) , this, SLOT( CloseFile() ) );

	QAction * fileSaveAsAction;
	fileSaveAsAction = new QAction( QIcon(ProgramPath+"images/filesaveas.png"), "Save &As...", this );
	fileSaveAsAction->setStatusTip("Save as...");
	connect( fileSaveAsAction, SIGNAL( triggered() ) , this, SLOT(SaveAsFile()  ) );

	QAction * WinTileAction;
	WinTileAction = new QAction( QIcon(ProgramPath+"images/tile.png"), "&Tile",this );
	WinTileAction->setStatusTip("Tile");
	connect( WinTileAction, SIGNAL( triggered() ) , this, SLOT( Tile() ) );

	QAction * WinCasAction;
	WinCasAction = new QAction( QIcon(ProgramPath+"images/cascade.png"), "&Cascade", this );
	WinCasAction->setStatusTip("Cascade");
	connect( WinCasAction, SIGNAL( triggered() ) , this, SLOT( Cascade() ) );
	
	QAction * InsertMD2Action;
	InsertMD2Action= new QAction( "MD2",this);
	InsertMD2Action->setStatusTip("Insert MD2 model");
	connect( InsertMD2Action, SIGNAL( triggered() ) , this, SLOT( InsertMD2() ) );
	
	QAction * InsertDXFAction;
	InsertDXFAction= new QAction("DXF",this);
	InsertDXFAction->setStatusTip("Insert DXF model");
	connect( InsertDXFAction, SIGNAL( triggered() ) , this, SLOT( InsertDXF() ) );
	
	QAction * ExportDXFAction;
	ExportDXFAction= new QAction( "DXF",  this);
	ExportDXFAction->setStatusTip("Export DXF model");
	connect( ExportDXFAction, SIGNAL( triggered() ) , this, SLOT( ExportDXF() ) );
	
	QAction * InsertQ3DAction;
	InsertQ3DAction= new QAction("3DQ", this);
	InsertQ3DAction->setStatusTip("Insert 3DQ model");
	connect( InsertQ3DAction, SIGNAL( triggered() ) , this, SLOT( InsertQ3DFile() ) );

	QAction * SetProgPathAction;
	SetProgPathAction= new QAction("Set &Path...", this);
	SetProgPathAction->setStatusTip("Set program ProgramPath");
	connect( SetProgPathAction, SIGNAL( triggered() ) , this, SLOT( SetProgPath() ) );

	QAction * QuitAction;
	QuitAction = new QAction( QIcon(ProgramPath+"images/fileclose.png"), "&Quit", this);
	QuitAction->setStatusTip("Quit");
	QuitAction->setShortcut(tr("CTRL+Q"));
	connect( QuitAction, SIGNAL( triggered() ) , qApp, SLOT(closeAllWindows()  ));
	
	QAction * FochoAction;
	FochoAction = new QAction( "&Focho", this);
	FochoAction->setShortcut(tr("F8"));addAction(FochoAction);
	connect( FochoAction, SIGNAL( triggered() ) , this, SLOT(sendsnapcmd()  ));
	
	QAction * CancelAction;
	CancelAction = new QAction(this);
	CancelAction->setShortcut(tr("Escape"));addAction(CancelAction);
	connect( CancelAction, SIGNAL( triggered() ) , this, SLOT(sendCancel()  ));
	
	QAction * TexDocsAction;
	TexDocsAction= new QAction( QIcon(ProgramPath+"images/texdocs.png"),"&Texture files",this);
	TexDocsAction->setStatusTip("Texture files");
	connect( TexDocsAction, SIGNAL( triggered() ) , this, SLOT( TexDocs() ) );
	/*//Draw Actions BEGIN//////////////////////////////////////////////////////////////////////////////////////////////*/
	QAction * DrawLineAction;
	DrawLineAction= new QAction( QIcon(ProgramPath+"images/trails.png"),"&Line",this);
	DrawLineAction->setStatusTip("Draw Line");
	connect( DrawLineAction, SIGNAL( triggered() ) , this, SLOT( DrawLine() ) );
	
	QAction * DrawPolyAction;
	DrawPolyAction= new QAction(QIcon(ProgramPath+"images/polygon.png"),"&Polygon",this);
	DrawPolyAction->setStatusTip("Draw polygon");
	connect( DrawPolyAction, SIGNAL( triggered() ) , this, SLOT( DrawPoly() ) );
	
	QAction * DrawTriangleAction;
	DrawTriangleAction= new QAction( QIcon(ProgramPath+"images/triangles.png"),"&Triangle",this);
	DrawTriangleAction->setStatusTip("Draw Triangle");
	connect( DrawTriangleAction, SIGNAL( triggered() ) , this, SLOT( DrawTriangle() ) );
	
	QAction * DrawCubeAction;
	DrawCubeAction= new QAction( QIcon(ProgramPath+"images/rectangle.png"),"&Rectangle",this);
	DrawCubeAction->setStatusTip("Draw rectangle");
	connect( DrawCubeAction, SIGNAL( triggered() ) , this, SLOT( DrawCube() ) );
	
	QAction * DrawArrayAction;
	DrawArrayAction= new QAction(QIcon(ProgramPath+"images/array.png"),"&Array",this);
	DrawArrayAction->setStatusTip("Draw Array");
	connect( DrawArrayAction, SIGNAL( triggered() ) , this, SLOT( DrawArray() ) );
	
	QAction * DrawRevolveAction;
	DrawRevolveAction= new QAction( QIcon(ProgramPath+"images/revolve.png"),"&Revolve",this);
	DrawRevolveAction->setStatusTip("Draw Revolve");
	connect( DrawRevolveAction, SIGNAL( triggered() ) , this, SLOT( Drawrevolve() ) );
	
	QAction * DrawExtrudeAction;
	DrawExtrudeAction= new QAction( QIcon(ProgramPath+"images/extrude.png"),"&Extrude",this);
	DrawExtrudeAction->setStatusTip("Draw Extrude");
	connect( DrawExtrudeAction, SIGNAL( triggered() ) , this, SLOT( DrawExtrude() ) );
	/*//Draw Actions END//////////////////////////////////////////////////////////////////////////////////////////////*/
	/*//View Actions BEGIN//////////////////////////////////////////////////////////////////////////////////////////////*/
	QAction * DrawZoomAction;
	DrawZoomAction= new QAction( QIcon(ProgramPath+"images/viewmag2.png"),"&Zoom",this);
	DrawZoomAction->setStatusTip("View Zoom");
	connect( DrawZoomAction, SIGNAL( triggered() ) , this, SLOT( ViewZoom() ) );
	
	QAction * DrawPanAction;
	DrawPanAction= new QAction( QIcon(ProgramPath+"images/pan.png"),"&Pan",this);
	DrawPanAction->setStatusTip("View Pan");
	connect( DrawPanAction, SIGNAL( triggered() ) , this, SLOT( ViewPan() ) );
	
	QAction * RotateViewAction;
	RotateViewAction= new QAction( QIcon(ProgramPath+"images/rotateview.png"),"&Rotate view",this);
	RotateViewAction->setStatusTip("Rotate View");
	connect( RotateViewAction, SIGNAL( triggered() ) , this, SLOT( ViewRotateView() ) );
	
	QAction * DistAction;
	DistAction= new QAction(QIcon(ProgramPath+"images/dist.png"),"&Distance",this);
	DistAction->setStatusTip("Distance");
	connect( DistAction, SIGNAL( triggered() ) , this, SLOT( ViewDist() ) );
	
	/*//View Actions END//////////////////////////////////////////////////////////////////////////////////////////////*/
	/*//Modify Actions BEGIN//////////////////////////////////////////////////////////////////////////////////////////////*/
	QAction * ModPropertiesAction;
	ModPropertiesAction= new QAction( QIcon(ProgramPath+"images/vopts.png"),"&Properties",this);
	ModPropertiesAction->setStatusTip("Properties");
	connect( ModPropertiesAction, SIGNAL( triggered() ) , this, SLOT( ModProperties() ) );
	
	QAction * ModEraseAction;
	ModEraseAction= new QAction( QIcon(ProgramPath+"images/erase.png"),"&Erase",this);
	ModEraseAction->setStatusTip("Erase");
	connect( ModEraseAction, SIGNAL( triggered() ) , this, SLOT( ModErase() ) );
	
	QAction * ModRotate3dAction;
	ModRotate3dAction= new QAction( QIcon(ProgramPath+"images/rotate3d.png"),"&Rotate 3d",this);
	ModRotate3dAction->setStatusTip("Rotate 3d");
	connect( ModRotate3dAction, SIGNAL( triggered() ) , this, SLOT( ModRotate3d() ) );
	
	QAction * ModSliceAction;
	ModSliceAction= new QAction(QIcon(ProgramPath+"images/slice.png"),"&Slice",this);
	ModSliceAction->setStatusTip("Slice");
	connect( ModSliceAction, SIGNAL( triggered() ) , this, SLOT( ModSlice() ) );
	
	QAction * ModSwitchAction;
	ModSwitchAction= new QAction( QIcon(ProgramPath+"images/switch.png"),"&Switch",this);
	ModSwitchAction->setStatusTip("Switch");
	connect( ModSwitchAction, SIGNAL( triggered() ) , this, SLOT( ModSwitch() ) );
	
	QAction * ModTrinormalsAction;
	ModTrinormalsAction= new QAction( QIcon(ProgramPath+"images/trinorm.png"),"&Normals Triangle",this);
	ModTrinormalsAction->setStatusTip("Normals Triangle");
	connect( ModTrinormalsAction, SIGNAL( triggered() ) , this, SLOT( ModTrinormals() ) );
	
	QAction * ModMnormalsAction;
	ModMnormalsAction= new QAction( QIcon(ProgramPath+"images/mnormals.png"),"&Mean normals",this);
	ModMnormalsAction->setStatusTip("Mean normals");
	connect( ModMnormalsAction, SIGNAL( triggered() ) , this, SLOT( ModMnormals() ) );
	
	/*//Modify Actions END//////////////////////////////////////////////////////////////////////////////////////////////*/
	QAction * AboutAction;
	AboutAction= new QAction("&About",this);
	AboutAction->setStatusTip("About");
	connect( AboutAction, SIGNAL( triggered() ) , this, SLOT( AboutFun() ) );
	
	QAction * ViewportPropAction;
	ViewportPropAction = new QAction(QIcon(ProgramPath+"images/vwprop.png"), "&Viewport Properties", this);
	ViewportPropAction->setStatusTip("Viewport Properties");
	ViewportPropAction->setShortcut(tr("Key_F10"));
	connect( ViewportPropAction, SIGNAL( triggered() ) , this, SLOT( ViewportProp() ) );

	QPixmap pix( 16, 16 );pix.fill( Qt::white);
	ActionColor = new QAction( pix, "&Color...", this);
	ViewportPropAction->setStatusTip("Color");
	connect( ActionColor, SIGNAL( triggered() ), this, SLOT( currentColor() ) );

 /*/MENU PRINCIPAL///////////////////////////////*/
//////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////*/

    QMenu *InsertItems = new QMenu( this );
	QMenu *ExportItems = new QMenu( this );
    
	QMenu *file;
	file = menuBar()->addMenu("&File");
	file->addAction(fileOpenAction);
	file->addAction(fileSaveAction);
	file->addAction(fileSaveAsAction);
	file->addAction(fileCloseAction);
	file->addAction(TexDocsAction);
	file->addSeparator();
	InsertItems=file->addMenu("&Insert");
    		InsertItems->addAction(InsertMD2Action);
			InsertItems->addAction(InsertDXFAction);
			InsertItems->addAction(InsertQ3DAction);
	ExportItems=file->addMenu("&Export");
			ExportItems->addAction(ExportDXFAction );
	file->addSeparator();
  	file->addAction(SetProgPathAction);
	file->addSeparator();
	file->addAction(QuitAction);

	QMenu *drawmenu;
	drawmenu = menuBar()->addMenu("&Draw");
	drawmenu->addAction(DrawLineAction);
	drawmenu->addAction(DrawTriangleAction);
	drawmenu->addAction(DrawPolyAction);
	drawmenu->addAction(DrawCubeAction);
	drawmenu->addSeparator();
	drawmenu->addAction(DrawArrayAction);
	drawmenu->addAction(DrawRevolveAction);
	drawmenu->addAction(DrawExtrudeAction);

	QMenu *view;
	view = menuBar()->addMenu("&View");
	view->addAction(DrawZoomAction);
	view->addAction(DrawPanAction);
	view->addAction(RotateViewAction);
	view->addSeparator();
	view->addAction(DistAction);
	
	QMenu *viewport;
	viewport = menuBar()->addMenu("&Viewport");
	viewport->addAction(ViewPortNewAction);
	viewport->addAction(ViewportPropAction);

	QMenu *modify;
	modify = menuBar()->addMenu("&Modify");
	modify->addAction(ModPropertiesAction);
	modify->addAction(ModEraseAction);
	modify->addAction(ModRotate3dAction);
	modify->addSeparator();
	modify->addAction(ModSliceAction);
	modify->addSeparator();
	modify->addAction(ModSwitchAction);
	modify->addAction(ModTrinormalsAction);
	modify->addAction(ModMnormalsAction);


	QMenu *windowsMenu;
	windowsMenu = menuBar()->addMenu("&Windows");
	windowsMenu->addAction(WinTileAction);
	windowsMenu->addAction(WinCasAction);

	QMenu *AboutMenu;
	AboutMenu = menuBar()->addMenu("&About");
	AboutMenu->addAction(AboutAction);

/*//TOOLBAR//////////////////////////////////////////////*/
//////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////*/
	QToolBar *ToolB;
	ToolB = addToolBar("File");
	ToolB->setIconSize(QSize(16,16));
	ToolB->addAction(fileOpenAction);
	ToolB->addAction(fileSaveAction);

	QToolBar *DrawTool;
	DrawTool = addToolBar("Draw");
	DrawTool->setIconSize(QSize(16,16));
	DrawTool->addAction(DrawLineAction);
	DrawTool->addAction(DrawTriangleAction);
	DrawTool->addAction(DrawPolyAction);
	DrawTool->addAction(DrawCubeAction);
	DrawTool->addAction(DrawArrayAction);
	DrawTool->addAction(DrawRevolveAction);
	DrawTool->addAction(DrawExtrudeAction);

	QToolBar *ToolMod;
	ToolMod = addToolBar("Modify");
	ToolMod->setIconSize(QSize(16,16));
	ToolMod->addAction(ActionColor);
	ToolMod->addAction(ModPropertiesAction);
	ToolMod->addAction(ModEraseAction);
	ToolMod->addAction(ModRotate3dAction);
	ToolMod->addAction(ModSliceAction);
	ToolMod->addAction(ModSwitchAction);
	ToolMod->addAction(ModTrinormalsAction);
	ToolMod->addAction(ModMnormalsAction);

	QToolBar *ViewTool;
	ViewTool = addToolBar("View");
	ViewTool->setIconSize(QSize(16,16));
	ViewTool->addAction(DrawZoomAction);
	ViewTool->addAction(DrawPanAction);
	ViewTool->addAction(RotateViewAction);
	ViewTool->addAction(DistAction);

	QToolBar *ViewportTool;
	ViewportTool = addToolBar("Viewport");
	ViewportTool->setIconSize(QSize(16,16));
	ViewportTool->addAction(ViewPortNewAction);
	ViewportTool->addAction(ViewportPropAction);	
	


/*/////////////////////////////////////////////////////*/
	
	
	
	hcmd->setGeometry( 0,0,100, 20 );
	

	QPalette newPalette = palette();
	newPalette.setColor(QPalette::Background, QColor::QColor(170,170,255));
	newPalette.setColor(QPalette::Foreground, QColor::QColor(0,0,0));
	newPalette.setColor(QPalette::Base, QColor::QColor(170,170,255));

	

	hHL->addWidget(Label1);
	hHL->addWidget(ecmd);
	hVc->addWidget(hcmd);
	hHL->setMargin(0);hHL->setSpacing(0);
	hVc->setMargin(0);hVc->setSpacing(0);
	WhVc->setLayout(hVc);

	
	s1->addWidget(ws);
	s1->addWidget(WhVc);

	ws->setScrollBarsEnabled( TRUE );
	
	
    Label1->setPalette(newPalette);
	ecmd->setPalette(newPalette);
	
	
	vboxLayout->addWidget(s1);
	vboxLayout->addLayout(hHL);
	vboxLayout->setMargin(0);
	vboxLayout->setSpacing(0);

	hcmd->setWindowIcon( QPixmap(ProgramPath+"images/console.png"));
	hcmd->setGeometry( 0,0,100, 20 );
	hcmd->currentcolor=currentcolor;
	hcmd->setDataL(&DataLV,&DataTV,MouseMode);


	

	QWidget *vb=new QWidget(this);
	vb->setLayout(vboxLayout);
	
    setCentralWidget(vb);
	
	connect( ecmd, SIGNAL(returnPressed()) , this, SLOT(sendCmd()  ));
	connect( ecmd, SIGNAL( lostFocus() ) , ecmd, SLOT(setFocus()  ));
	connect( hcmd, SIGNAL( instru(const QString&) ), Label1, SLOT( setText(const QString& )) );
	connect( hcmd, SIGNAL( SetMouseMode(int )), this, SLOT( SetMouseMode(int )) );
	connect( hcmd, SIGNAL( ZoomVar(float )), this, SLOT( ZoomVar(float )) );

	/*/CURSOR//////////////////////////////////////////////////////////////////*/
	QBitmap curb1(32,32);
	QBitmap curb2(32,32);
	curb1.fill(QColor(255,255,255));
	curb2.fill(QColor(255,255,255));
	QPainter p;
	
	p.begin( &curb1 );p.drawLine(16,0,16,31);p.drawLine(0,16,31,16);p.end();
	gencursor[MOUSEMODE_PICKPOINT]=QCursor(curb2,curb1);
	p.begin( &curb1 );p.setBrush(Qt::NoBrush);p.drawRect(13, 13,6,6 );p.end();
	gencursor[MOUSEMODE_SELECT]=QCursor(curb2,curb1);	
	gencursor[MOUSEMODE_ZOOM]=QCursor(QBitmap(ProgramPath+"images/zoomi.png"),QBitmap(ProgramPath+"images/zoomim.png"));
	gencursor[MOUSEMODE_PAN]=QCursor(QBitmap(ProgramPath+"images/pani.png"),QBitmap(ProgramPath+"images/panm.png"));
	gencursor[MOUSEMODE_ROTATE]=QCursor(QBitmap(ProgramPath+"images/rotatei.png"),QBitmap(ProgramPath+"images/rotatem.png"));
	
	/*///////////////////////////////////////////////////////////////////*/

	ecmd->setFocus();
   	statusBar()->showMessage("Ready",2000);
    setWindowTitle("AutoQ3D");
	setWindowIcon( QPixmap(ProgramPath+"AutoQ3D.png") );

	hcmd->setMinimumSize(hcmd->minimumSizeHint().width(),50);
	hcmd->setSizePolicy(QSizePolicy(QSizePolicy::Minimum,QSizePolicy::Ignored));
	
}

/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::closeEvent(QCloseEvent*)
{
	exit();
}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::exit()
{
	/*QSettings settings;
	settings.writeEntry( APP_KEY + "ProgramPath",ProgramPath );*/

	QApplication::exit( 0 );
}



CadForm::~CadForm(){


}

void CadForm::ZoomVar(float var){
	GLApp *win=(GLApp*)ws->activeWindow();
	win->GLThisArea->ZoomVar(var);

}

/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::SetMouseMode(int mmode){
	
	int i;
	MouseMode[0]=mmode;
	
	QWidgetList windoL=ws->windowList(QWorkspace::CreationOrder);
	if ( windoL.count() ) {
        	for (i = 0; i < int(windoL.count()); i++) {
			GLApp *win =(GLApp*) windoL.at( i );
			win->GLThisArea->setCursor(gencursor[mmode]);
			win->GLThisArea->UpdateMouseMove();
		}
	}
	
	
}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::AboutFun(){
	About* w = new About;
	w->exec();
	w->setAttribute(Qt::WA_DeleteOnClose);
	w->close();

}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::TexDocs(){
	int i;
	QString filedir(filename);
	filedir.truncate(filedir.lastIndexOf( "/" ));
	

	TexForm* w = new TexForm( 0,0,hcmd->StrTex,filedir);
	if(w->exec()){
		hcmd->StrTex.clear();
		QMessageBox::information(this,"Information","The program will close all viewports to update textures files", "&OK");
		for(i=0;i<w->srtlist2.count();i++){
			hcmd->StrTex<<w->srtlist2.at(i);
			/*hcmd->SendText(*w->srtlist2.at(i));*/
		}
		
			QWidgetList windoL=ws->windowList(QWorkspace::CreationOrder);
		if ( windoL.count() ) {
	        	for (i = 0; i < windoL.count(); i++) {
				GLApp *win =(GLApp*) windoL.at( i );
				win->close();
			}
		}
		
	}
	w->setAttribute(Qt::WA_DeleteOnClose);
	w->close();

}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/

void CadForm::currentColor(){

	QColor col = QColorDialog::getColor( QColor ( (int)currentcolor[0]*255, (int)currentcolor[1]*255, (int)currentcolor[2]*255), this );
	if ( !col.isValid() )
	return;

	QPixmap pix( 16, 16 );pix.fill( col );
	ActionColor->setIcon( pix );
	currentcolor[0]=(float)col.red()/255.0;currentcolor[1]=(float)col.green()/255.0;currentcolor[2]=(float)col.blue()/255.0;
 
}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::DrawLine(){hcmd->processcmd("line");}
void CadForm::DrawTriangle(){hcmd->processcmd("triangle");}
void CadForm::DrawPoly(){hcmd->processcmd("polygon");}
void CadForm::DrawCube(){hcmd->processcmd("rectangle");}
void CadForm::Drawrevolve(){hcmd->processcmd("revolve");}
void CadForm::DrawArray(){hcmd->processcmd("array");}
void CadForm::DrawExtrude(){hcmd->processcmd("extrude");}

void CadForm::ViewZoom(){hcmd->processcmd("'zoom");}
void CadForm::ViewPan(){hcmd->processcmd("'pan");}
void CadForm::ViewRotateView(){hcmd->processcmd("'rotateview");}
void CadForm::ViewDist(){hcmd->processcmd("dist");}

void CadForm::ModProperties(){hcmd->processcmd("properties");}
void CadForm::ModErase(){hcmd->processcmd("erase");}
void CadForm::ModRotate3d(){hcmd->processcmd("rotate3d");}
void CadForm::ModSlice(){hcmd->processcmd("slice");}
void CadForm::ModSwitch(){hcmd->processcmd("switch");}
void CadForm::ModTrinormals(){hcmd->processcmd("trinormals");}
void CadForm::ModMnormals(){hcmd->processcmd("mnormals");}
void CadForm::sendsnapcmd(){hcmd->processcmd("'snap");}

/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/

void CadForm::UpdateAll(){

	int i;
	QWidgetList windoL=ws->windowList(QWorkspace::CreationOrder);
	if ( windoL.count() ) {
        	for (i = 0; i < int(windoL.count()); i++) {
			GLApp *win =(GLApp*) windoL.at( i );
			win->GLThisArea->updateGL();
		}
	}

}


/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
GLApp* CadForm::newViewPort()
{
	GLApp* w = new GLApp(0, Qt::Widget,hcmd->StrTex);
	ws->addWindow(w);
	w->setAttribute(Qt::WA_DeleteOnClose);
	connect( w->GLThisArea, SIGNAL( setCmdData(dataformat) ) ,hcmd, SLOT(setCmdData(dataformat)  ));
	connect( w->GLThisArea, SIGNAL( processcmd(QString) ) ,hcmd, SLOT(processcmd(QString)  ));
	connect( w->GLThisArea, SIGNAL( SendText(QString) ) ,hcmd, SLOT(SendText(QString)  ));
	connect( w->GLThisArea, SIGNAL( UpdateAll() ) ,this, SLOT(UpdateAll()  ));
	connect( w->GLThisArea, SIGNAL( message(const QString&, int) ), statusBar(), SLOT( showMessage(const QString&, int )) );
    
	w->GLThisArea->setDataL(&DataLV,&DataTV,&hcmd->DataSV,&hcmd->DataSLV,&hcmd->DataSTV,MouseMode);
	w->GLThisArea->rtlines=hcmd->rtlines;
	w->GLThisArea->GriPoint=hcmd->GriPoint;
	w->GLThisArea->currentpoint=hcmd->currentpoint;
	w->GLThisArea->SnaPoint=hcmd->SnaPoint;
	w->GLThisArea->SnapMode=hcmd->SnapMode;
	w->GLThisArea->SnapFlags=hcmd->SnapFlags;
	w->GLThisArea->rtcolor=hcmd->rtcolor;
	w->GLThisArea->rtcolor1=hcmd->rtcolor1;
	w->GLThisArea->GripEditMode=hcmd->GripEditMode;
	w->GLThisArea->currentcolor=currentcolor;
	w->GLThisArea->cscalarf=hcmd->cscalarf;
	w->GLThisArea->cscalari=hcmd->cscalari;
	
	w->setWindowTitle(QString::QString("ViewPort-%1").arg(numDoc++));w->setWindowIcon( QPixmap(ProgramPath+"images/document.png") );
	w->show();
	QWidgetList windoL=ws->windowList(QWorkspace::CreationOrder);
	if ( windoL.count()==1){
		w->showMaximized();
	}else{
		
		ws->tile();
	}
	
	
	UpdateAll();
	SetMouseMode(MOUSEMODE_SELECT);
	return w;
}



/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/


void CadForm::OpenFile()
{

	int i=0;
	QWidgetList windoL=ws->windowList(QWorkspace::CreationOrder);
	if ( windoL.count() ) {
        	for (i = 0; i < int(windoL.count()); i++) {
			GLApp *win =(GLApp*) windoL.at( i );
			win->close();
		}
	}
	DataLV.resize(0);DataTV.resize(0);hcmd->DataSV.resize(0);

	QString fn = QFileDialog::getOpenFileName(this,QString(""),filename,"3D model(*.3dq)" );
	float version;
	QString tempo;
	int numlines,numtriangles,numtextures;
	i=0;
	if ( !fn.isEmpty() ) {
	
	QFile f( fn );
	
	if ( !f.open( QIODevice::ReadOnly ) ){
		
		QMessageBox::critical( this,tr("Fatal error"),QString("Open failed"),tr("OK") );
		f.close();
	
	}else{
		filename=fn;
		
		QTextStream t(&f);
		if(!sscanf(t.readLine().toAscii(),"ver3data=%f",&version)){
			f.close();
			QMessageBox::critical( this,tr("Fatal error"),QString("Open failed"),tr("OK") );
			return;
		}
		sscanf(t.readLine().toAscii(),"numlines=%d",&numlines);
		sscanf(t.readLine().toAscii(),"numtriangles=%d",&numtriangles);
		sscanf(t.readLine().toAscii(),"numtextures=%d",&numtextures);
		/*sscanf(t.readLine(),"numtextures=%d",&numtextures);*/
		setWindowTitle("AutoQ3D--"+filename);
		QString filedir(filename);
		filedir.truncate(filedir.lastIndexOf( "/" ));
		hcmd->StrTex.clear();
		for(i=0;i<numtextures;i++){
			hcmd->StrTex+=filedir+"/"+t.readLine();
		}
		
		/*while(!t.atEnd()){*/
		for(i=0;i<numlines;i++){
			DataLV.resize(i+1);
			sscanf(t.readLine().toAscii(),"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",&DataLV[i].v1[0],&DataLV[i].v1[1],&DataLV[i].v1[2],&DataLV[i].v2[0],&DataLV[i].v2[1],&DataLV[i].v2[2],&DataLV[i].c1[0],&DataLV[i].c1[1],&DataLV[i].c1[2],&DataLV[i].c2[0],&DataLV[i].c2[1],&DataLV[i].c2[2]);
		}
		
		for(i=0;i<numtriangles;i++){
			DataTV.resize(i+1);
			sscanf(t.readLine().toAscii(),"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d,%f,%f,%f,%f,%f,%f",&DataTV[i].v1[0],&DataTV[i].v1[1],&DataTV[i].v1[2],&DataTV[i].v2[0],&DataTV[i].v2[1],&DataTV[i].v2[2],&DataTV[i].v3[0],&DataTV[i].v3[1],&DataTV[i].v3[2],&DataTV[i].c1[0],&DataTV[i].c1[1],&DataTV[i].c1[2],&DataTV[i].c2[0],&DataTV[i].c2[1],&DataTV[i].c2[2],&DataTV[i].c3[0],&DataTV[i].c3[1],&DataTV[i].c3[2],&DataTV[i].n1[0],&DataTV[i].n1[1],&DataTV[i].n1[2],&DataTV[i].n2[0],&DataTV[i].n2[1],&DataTV[i].n2[2],&DataTV[i].n3[0],&DataTV[i].n3[1],&DataTV[i].n3[2],&DataTV[i].texture,&DataTV[i].t1[0],&DataTV[i].t1[1],&DataTV[i].t2[0],&DataTV[i].t2[1],&DataTV[i].t3[0],&DataTV[i].t3[1]);
		}
		f.close();
		
		newViewPort();
		
	}
	
	
    }else {
	statusBar()->showMessage( "Loading aborted", 2000 );
    }
}

/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::SaveFile()
{
	int i;
	QString tempo;
	if (filename.isEmpty()){SaveAsFile();return;}
    
	QFile f(filename);
	if ( !f.open( QIODevice::WriteOnly ) ) {QMessageBox::critical(this,"Information",QString("Could not write to %1").arg(filename), "&OK");return;}
	
	QTextStream stream( &f );stream<<"ver3data=1.0\nnumlines="<<DataLV.size()<<"\nnumtriangles="<<DataTV.size()<<"\nnumtextures="<<hcmd->StrTex.count()<<"\n";
	
	for(QStringList::Iterator it=hcmd->StrTex.begin();it!=hcmd->StrTex.end();++it){
		tempo=*it;
		tempo.remove( 0,tempo.lastIndexOf( "/" )+1);
		stream<<tempo<<"\n";
	}
	
	for(i=0;i<DataLV.size();i++){
		stream<<DataLV[i].v1[0]<<","<<DataLV[i].v1[1]<<","<<DataLV[i].v1[2]<<","<<DataLV[i].v2[0]<<","<<DataLV[i].v2[1]<<","<<DataLV[i].v2[2]<<","<<DataLV[i].c1[0]<<","<<DataLV[i].c1[1]<<","<<DataLV[i].c1[2]<<","<<DataLV[i].c2[0]<<","<<DataLV[i].c2[1]<<","<<DataLV[i].c2[2]<<"\n";
	}
	for(i=0; i<DataTV.size();i++){
		stream<<DataTV[i].v1[0]<<","<<DataTV[i].v1[1]<<","<<DataTV[i].v1[2]<<","<<DataTV[i].v2[0]<<","<<DataTV[i].v2[1]<<","<<DataTV[i].v2[2]<<","<<DataTV[i].v3[0]<<","<<DataTV[i].v3[1]<<","<<DataTV[i].v3[2]<<","<<DataTV[i].c1[0]<<","<<DataTV[i].c1[1]<<","<<DataTV[i].c1[2]<<","<<DataTV[i].c2[0]<<","<<DataTV[i].c2[1]<<","<<DataTV[i].c2[2]<<","<<DataTV[i].c3[0]<<","<<DataTV[i].c3[1]<<","<<DataTV[i].c3[2]<<","<<DataTV[i].n1[0]<<","<<DataTV[i].n1[1]<<","<<DataTV[i].n1[2]<<","<<DataTV[i].n2[0]<<","<<DataTV[i].n2[1]<<","<<DataTV[i].n2[2]<<","<<DataTV[i].n3[0]<<","<<DataTV[i].n3[1]<<","<<DataTV[i].n3[2]<<","<<DataTV[i].texture<<","<<DataTV[i].t1[0]<<","<<DataTV[i].t1[1]<<","<<DataTV[i].t2[0]<<","<<DataTV[i].t2[1]<<","<<DataTV[i].t3[0]<<","<<DataTV[i].t3[1]<<"\n";
	}

	
	f.close();
	setWindowTitle("AutoQ3D--"+filename);
	hcmd->SendText("--Save file--");
}

/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::SaveAsFile()
{
	QString fn = QFileDialog::getSaveFileName( this,QString(""),filename, QString("") );
	if ( !fn.isEmpty() ) {
	if((int)(fn.length()-4)==fn.lastIndexOf(".3dq")){
		filename = fn;
		}else{
			filename = fn+".3dq";
		}
		SaveFile();
	} else {
		statusBar()->showMessage( "Saving aborted", 2000 );
	}

}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/


void CadForm::SetProgPath()
{
QString tempo;
QSettings settings("AutoQ3D","AutoQ3D");
tempo= QFileDialog::getExistingDirectory(this,"Set Program Path",ProgramPath,QFileDialog::ShowDirsOnly);
if(!tempo.isNull()){
		ProgramPath =tempo;

		settings.setValue( "MainWindow/ProgramPath",ProgramPath );
		QMessageBox::information(this,"Information","Restart for load the changes", "&OK");
	}
}


/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::sendCmd(){
	dataformat info;int i;
	info.format=0;
	if((i=ecmd->text().count(','))){
		if(i==2){
			if(ecmd->text().contains('@')){
				info.format=1;i=sscanf(ecmd->text().toAscii(),"@%f,%f,%f",&info.v[0],&info.v[1],&info.v[2]);
			}else{i=sscanf(ecmd->text().toAscii(),"%f,%f,%f",&info.v[0],&info.v[1],&info.v[2]);}
			hcmd->SendText(ecmd->text());
			ecmd->setText("");
			hcmd->setCmdData(info);
		}else{hcmd->SendText("***Error***");}
	}else{
			QDoubleValidator *vale=new QDoubleValidator(this);
			QString str(ecmd->text());
			int temp=0;
			if(vale->validate(str,temp)==QValidator::Acceptable){
				info.format=2;
				info.v[0]=ecmd->text().toDouble();
				hcmd->SendText(ecmd->text());
				ecmd->setText("");
				hcmd->setCmdData(info);
				UpdateAll();
				return;
			}
	
	hcmd->processcmd(ecmd->text());ecmd->setText("");}
	UpdateAll();
}

void CadForm::InsertQ3DFile()
{

	int i=0,j=0;
	QWidgetList windoL=ws->windowList(QWorkspace::CreationOrder);
	if ( windoL.count() ) {
        	for (i = 0; i < int(windoL.count()); i++) {
			GLApp *win =(GLApp*) windoL.at( i );
			win->close();
		}
	}
	
	
	QString fn = QFileDialog::getOpenFileName(this,QString("Insert 3dq file"),filename,"3D model(*.3dq)" );
	float version;
	QString tempo;
	int numtexold;
	int numlines,numtriangles,numtextures;
	i=0;
	if ( !fn.isEmpty() ) {
	
	QFile f( fn );
	
	if ( !f.open( QIODevice::ReadOnly ) ){
		
		QMessageBox::critical( this,tr("Fatal error"),QString("Open failed"),tr("OK") );
		f.close();
	
	}else{
		filename=fn;
		
		QTextStream t(&f);
		if(!sscanf(t.readLine().toAscii(),"ver3data=%f",&version)){
			f.close();
			QMessageBox::critical( this,tr("Fatal error"),QString("Open failed"),tr("OK") );
			return;
		}
		sscanf(t.readLine().toAscii(),"numlines=%d",&numlines);
		sscanf(t.readLine().toAscii(),"numtriangles=%d",&numtriangles);
		sscanf(t.readLine().toAscii(),"numtextures=%d",&numtextures);
		/*sscanf(t.readLine(),"numtextures=%d",&numtextures);*/
		setWindowTitle("AutoQ3D--"+filename);
		QString filedir(filename);
		filedir.truncate(filedir.lastIndexOf( "/" ));
		numtexold=hcmd->StrTex.count();
		for(i=0;i<numtextures;i++){
			hcmd->StrTex+=filedir+"/"+t.readLine();
		}
		
		/*while(!t.atEnd()){*/
		for(i=0;i<numlines;i++){
			j=DataLV.size();DataLV.resize(j+1);
			sscanf(t.readLine().toAscii(),"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f",&DataLV[j].v1[0],&DataLV[j].v1[1],&DataLV[j].v1[2],&DataLV[j].v2[0],&DataLV[j].v2[1],&DataLV[j].v2[2],&DataLV[j].c1[0],&DataLV[j].c1[1],&DataLV[j].c1[2],&DataLV[j].c2[0],&DataLV[j].c2[1],&DataLV[j].c2[2]);
		}
		
		for(i=0;i<numtriangles;i++){
			j=DataTV.size();DataTV.resize(j+1);
			sscanf(t.readLine().toAscii(),"%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d,%f,%f,%f,%f,%f,%f",&DataTV[j].v1[0],&DataTV[j].v1[1],&DataTV[j].v1[2],&DataTV[j].v2[0],&DataTV[j].v2[1],&DataTV[j].v2[2],&DataTV[j].v3[0],&DataTV[j].v3[1],&DataTV[j].v3[2],&DataTV[j].c1[0],&DataTV[j].c1[1],&DataTV[j].c1[2],&DataTV[j].c2[0],&DataTV[j].c2[1],&DataTV[j].c2[2],&DataTV[j].c3[0],&DataTV[j].c3[1],&DataTV[j].c3[2],&DataTV[j].n1[0],&DataTV[j].n1[1],&DataTV[j].n1[2],&DataTV[j].n2[0],&DataTV[j].n2[1],&DataTV[j].n2[2],&DataTV[j].n3[0],&DataTV[j].n3[1],&DataTV[j].n3[2],&DataTV[j].texture,&DataTV[j].t1[0],&DataTV[j].t1[1],&DataTV[j].t2[0],&DataTV[j].t2[1],&DataTV[j].t3[0],&DataTV[j].t3[1]);
			if(DataTV[j].texture){
				DataTV[j].texture+=numtexold;
			}
		}
		f.close();
		
		
	}
	
	
    }else {
	statusBar()->showMessage( "Loading aborted", 2000 );
    }
}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/

void CadForm::ViewportProp(){

	GLApp *win =(GLApp*)ws->activeWindow();
	QString str;
	
	if(win){
		Viewprop* w = new Viewprop(this);
		w->EFar->setText(QString("%1").arg(win->GLThisArea->zFar));
		w->ENear->setText(QString("%1").arg(win->GLThisArea->zNear));
		w->NSize->setText(QString("%1").arg(win->GLThisArea->NS));
		w->chBN->setChecked(win->GLThisArea->ShowNormals);
		w->chBW->setChecked(!win->GLThisArea->RenderMode);
		if(w->exec()){
			str=w->EFar->text();win->GLThisArea->zFar=str.toFloat();
			str=w->ENear->text();win->GLThisArea->zNear=str.toFloat();
			str=w->NSize->text();win->GLThisArea->NS=str.toFloat();
			win->GLThisArea->ShowNormals=w->chBN->isChecked();
			win->GLThisArea->PublicUpdate();
			win->GLThisArea->RenderMode=!w->chBW->isChecked();
		
		}
		w->setAttribute(Qt::WA_DeleteOnClose);
		w->close();
	}
}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::sendCancel(){
	hcmd->processcmd("**Cancel**");
	ecmd->setText("");
	UpdateAll();

}
/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/
void CadForm::Tile()
{
	ws->tile();
}

/*/////////////////////////////////////////////////////*/
/*/////////////////////////////////////////////////////*/

void CadForm::Cascade()
{
	ws->cascade();
}

void CadForm::CloseFile(){
	int i=0;
	QWidgetList windoL=ws->windowList(QWorkspace::CreationOrder);
	if ( windoL.count() ) {
        	for (i = 0; i < int(windoL.count()); i++) {
			GLApp *win =(GLApp*) windoL.at( i );
			win->close();
		}
	}
	DataLV.resize(0);DataTV.resize(0);hcmd->DataSV.resize(0);
	newViewPort();
}

